package com.constructiveproof.hackertracker.auth.strategies

import org.scalatra.ScalatraBase
import javax.servlet.http.{HttpServletResponse, HttpServletRequest}
import org.scalatra.auth.ScentryStrategy
import com.constructiveproof.hackertracker.models.User
import org.slf4j.LoggerFactory


class UserPasswordStrategy(protected val app: ScalatraBase)
  (implicit request: HttpServletRequest, response: HttpServletResponse)
  extends ScentryStrategy[User] {

  override def name: String = "UserPassword"

  val logger = LoggerFactory.getLogger(getClass)

  private def login = app.params.getOrElse("login", "")
  private def password = app.params.getOrElse("password", "")

  /***
    * Determine whether the strategy should be run for the current request.
    */
  override def isValid(implicit request: HttpServletRequest) = {
    logger.info("UserPasswordStrategy: determining isValid: " + (login != "" && password != "").toString())
    login != "" && password != ""
  }


  /**
   *  In real life, this is where we'd consult our data store, asking it whether the user credentials matched
   *  any existing user. Here, we'll just check for a known login/password combination and return a user if
   *  it's found.
   */
  def authenticate()(implicit request: HttpServletRequest, response: HttpServletResponse): Option[User] = {
    logger.info("UserPasswordStrategy: attempting authentication")

    if(login == "foo" && password == "foo") {
      logger.info("UserPasswordStrategy: login succeeded")
      Some(User("foo"))
    } else {
      logger.info("UserPasswordStrategy: login failed")
      None
    }
  }

  /**
   * What should happen if the user is currently not authenticated?
   */
  override def unauthenticated()(implicit request: HttpServletRequest, response: HttpServletResponse) {
    app.redirect("/sessions/new")
  }
}